function [ainf, dinf, minf, kinf, manz]=loadatisa(filename, mode, start, ende, zykl);
%[ainf, dinf, minf, kinf, manz]=loadmes(filename, mode, start, ende); 
%Funktion zum Einlesen 
%Rudi 13.6.2003      - V3.0 Kanalname: Kanal+KaNr. wenn Kanalname==''
%Rudi 19.3.2003      - schneller Daten lesen V3.0
%                    - mehrere Fehler im Dateizugriff beseitigt V3.0
%Stand 01.10.2002
%Revision 01.10.2002 - Leseroutine fr aktuelle GJB-Daten
%Revision 15.12.1999 - zusaetzlicher Mode zum besseren Markerhandling
%Revision 22.02.1999 - zusaetzlicher Mode zum besseren Markerhandling; Markeranzahl pro Markertyp
%                      mehrere Fehler im Dateizugriff beseitigt
%Revision 15.02.1999 - zusaetzlicher Mode zum besseren Markerhandling
%Revision 10.02.1999 - Fehler bei Mode-Angabe beseitigt
%Revision 01.02.1999 - Auslesemodus zugefuegt
%Revision 31.01.1999 - ATISA 3.0-Single-Format unterstuetzt
%Revision 23.11.1998 - mehr als 127 Kanaele moeglich
%Folgende Formate werden z.Z.unterstuetzt:
%	- ATISA 3.0-Longfile mit fester Abtastfrequenz
%	- ATISA 3.0-Analysefile
%	- ATISA 3.0-Singlefile
%	- ATISA 2.10-Files
%	- ATISA MC80-Files
%Als Rueckgabeparameter werden folgende Datei-Informationen bereitgestellt:
%	- ainf - Allgemeine Datei-Informationen
%			ainf(1)-Dateityp (1-AT3.0-Longfile, 2-AT3.0-Analyse, 3-AT3.0-Single usw.)
%				wenn Typ=100, dann unbekanntes Format
%			ainf(2)-Kanalzahl
%			ainf(3)-Abtastfrequenz
%			ainf(4)-Messzyklen
%			ainf(5)-Markeranzahl
%	- dinf - Messdaten als Realzahlen
%			dinf(i,j)=Messwert	
%			     j - Kanal
%			     i - Messzyklus		
%	- minf - Markerinformationen
%			minf(i,1) - Markertyp
%			minf(i,2) - Markerposition in Messpunkten
%
%	- kinf - kanalbezogene Informationen
%			nur bei AT3.0-Files
%  - manz - Markerhufigkeiten
%        Vektor von 40 Werten, Anzahl des entsprechenden Markertyps an entsprechender Vektorstelle, 
%        z.B. manz(3)=3 bedeutet, da Markertyp 3 insgesamt 3x im File vorhanden ist
%mode - Angabe des Arbeitsmodes
%    ohne Angabe - Auslesen aller Dateiinformationen einschlielich Daten
%    Mode = 0    - nur Anzeige der wichtigsten Headerinformationen + Markerinformationen
%    Mode = 1    - in dinf werden nur die Daten ausgelesen, die zwischen start und ende-Parameter bergeben wurden
%                  (Angabe in Mewerten) 
%                  z.B. Start=312   Ende=519   , alle Werte zwischen 312 und 519
%    Mode = 2    - in dinf werden nur die Daten ausgelesen, die von Start bis Start+Ende-Parameter liegen 
%                  (Achtung: Ende enthlt Anzahl der Mewerte!!)
%                  (Angabe in Mewerten) 
%                  z.B. Start=312   Ende=100   , alle Werte von 312 bis 412
%    Mode = 3    - in dinf werden nur die Daten ausgelesen, die zwischen start und ende-Marker liegen
%                  bergabe der entsprechenden Marker-Nummern
%                  z.B. Start=3  Ende=5   alle Werte von Marker Nr.3 bis Marker Nr.5
%    Mode = 4    - in dinf werden nur die Daten ausgelesen, die bei Start-Marker beginnen und Ende Werte umfassen
%                  z.B. Start=3  Ende=250   ,250 Werte ab Marker Nr.3 
%    Mode = 5    - in dinf werden nur die Daten ausgelesen, die zwischen dem zykl. Vorkommen von start und Ende-Marker liegen
%                  z.B. Start=3  Ende=4  zykl=3 , 
%                  alle Werte zwischen dem 3.Vorkommen von Marker Typ 3 und dem 3.Vorkommen von Marker Typ 4 
%    Mode = 6    - in dinf werden nur die Daten ausgelesen, die bei dem zykl. Vorkommen von Start-Marker beginnen 
%                  und Ende Werte umfassen 
%                  z.B. Start=3  Ende=512  zykl=3 , alle Werte zwischen dem 3.Vorkommen von Marker Typ 3 +512 Werte
%
%!!!Achtung!!! nur Mode 5 und 6 beruecksichtigen den Markertyp, alle anderen Modi greifen auf die laufende Nr. des Markers zu


if (nargin<1)
  error('Requires a filename as argument.');
end;

if nargin==1
   mode=10;
end;   

if (nargin==2)&(mode~=0)
   error('Not enough parameters');
  elseif (nargin>1)&(nargin<4)&(mode~=0)
    error('Not enough parameters');
end;   

if ((mode==5)|(mode==6))&(nargin~=5)
   error('Not enough parameters');
  elseif (nargin>1)&(nargin<4)&(mode~=0)
    error('Not enough parameters');
end;   


if (isstr(filename)~=1)
	error('Requires a string filename as an argument.');
end;

if (isempty(findstr(filename,'.'))==1)
	filename=[filename,'.mes'];
end;


fid=fopen(filename,'rb','l');
if (fid==-1)
	error(['Error opening ',filename,' for input.']);
end;

bfType=fread(fid,4,'char');
char(abs(bfType));	

manz=zeros(65535,1);
dinf=zeros(1,1);
if nargin<3
  start=0;
  ende=0;
end;  


if (bfType~=['V';'3';'.';'0'])|(bfType~=['V';'2';'.';'1'])
  ainf(1)=100;
  h=fseek(fid,16384,-1);
  if h~=0
    fclose(fid);
    error('Error in file.');
  end;
  bfType=fread(fid,1,'int8'); 
  bfType=fread(fid,18,'char');
  h=str2num(setstr(bfType)');
  if isempty(h)
    fclose(fid);
    error('Error in file.');
  else
    ainf(1)=4;
  end;          
  kinf(1)=h;	
  bfType=fread(fid,1,'int8');  

  for i=2:16
    bfType=fread(fid,18,'char');
    kinf(i)=str2num(setstr(bfType)');
    bfType=fread(fid,1,'int8');
  end;

  for i=1:16
    bfType=fread(fid,18,'char');
    kinf(16+i)=str2num(setstr(bfType)');
    bfType=fread(fid,1,'int8');
  end;

  bfType=fread(fid,18,'char');
  ainf(2)=str2num(setstr(bfType)');
  bfType=fread(fid,1,'int8');
  bfType=fread(fid,18,'char');
  ainf(4)=str2num(setstr(bfType)');
  bfType=fread(fid,1,'int8');
  bfType=fread(fid,18,'char');
  ainf(3)=1/(str2num(setstr(bfType)')*406.90104e-09);
  bfType=fread(fid,1,'int8');
  ainf(5)=0;
  if (nargin==1)|(mode>2)
     start=1;
     ende=512;
  end   
  if mode==2
     ende=start+ende-1;
     if ende>512
        ende=512;
     end   
  end   
  if start<1
     start=1;
  end;
  if start>ende
     hilfv=ende;
     ende=start;
     start=hilfv;
  end;   
  h=fseek(fid,0,-1);
  if (nargin==1)|(mode~=0)
    for i=1:16
       bfType=fread(fid,512,'int16');
       for j=start:ende
          dinf(j-start+1,i)=bfType(j)*kinf(i)/32767;
      end;
    end;
  end;
else
  if (bfType==['V';'3';'.';'0'])		%ATISA3.0-File
   % hh=dir(which(filename));            %Rudi which=>   ??? Illegal right hand side in assignment. Too many elements.  
    hh=dir(filename);                    %... 
    flen=hh.bytes;      
    bfType=fread(fid,12,'char');
    filepos=fread(fid,4,'int32');
   
    [sfilepos,fileindex]=sort(filepos);    %Rudi
    dlen=flen-filepos(1);
    for i=1:3
       if fileindex(i)==1 
          dlen=sfilepos(i+1)-sfilepos(i); 
       end
    end; 
    %if filepos(1)<filepos(2)
      %h=fseek(fid,filepos(2),-1);
    %else
      %h=0;   
    %end;  
    h=fseek(fid,filepos(2),-1);         
    if isempty(h)
      fclose(fid);
      error('Error in file.');
    end;
    %bfType=fread(fid,1,'uint8');
    bfType=fread(fid,1,'uint16');     %Rudi  
    hilf(1)=bfType;
    %bfType=fread(fid,1,'int16');
    bfType=fread(fid,1,'int8');     %Rudi
    bfType=fread(fid,18,'char');
    hstr=setstr(bfType)';
    hilf(2)=str2num(hstr);
    bfType=fread(fid,1,'long');
    hilf(3)=bfType;
    ainf(2)=hilf(1);    
    %if hilf(3)==0                                   %Rudi
        %hilf(3)=(flen-32-35-203*ainf(2))/ainf(2)/2; 
    %end;    
    bfType=fread(fid,5,'int16');
    hilf(4:8)=bfType;
    if hilf(6)==0
      ainf(1)=1; 
      if hilf(3)==0                         %Rudi
         hilf(3)=dlen/ainf(2)/2;
      end;  
      ainf(4)=hilf(3);
    elseif hilf(6)==2
      ainf(1)=2;  
      ainf(4)=hilf(8);    
    elseif hilf(6)==3
      ainf(1)=3; 
      if hilf(3)==0                     %Rudi
         hilf(3)=dlen/ainf(2)/4;
      end;  
      ainf(4)=hilf(3);      
    else 
      ainf(1)=100;
    end;	
    ainf(3)=0;
    for i=1:ainf(2)
      bfType=fread(fid,1,'int16');
      kinf(i,1)=bfType;
      bfType=fread(fid,1,'int16');
      kinf(i,2)=bfType;
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,18,'char');
      hstr=setstr(bfType)';
      kinf(i,3)=str2num(hstr);
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,18,'char');
      hstr=setstr(bfType)';
      kinf(i,4)=str2num(hstr);
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,18,'char');
      hstr=setstr(bfType)';
      kinf(i,5)=str2num(hstr);
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,18,'char');
      hstr=setstr(bfType)';
      kinf(i,6)=str2num(hstr);
      bfType=fread(fid,1,'int16');
      kinf(i,7)=bfType;
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,18,'char');
      hstr=setstr(bfType)';
      kinf(i,8)=str2num(hstr);
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,18,'char');
      hstr=setstr(bfType)';
      kinf(i,9)=str2num(hstr);
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,18,'char');
      hstr=setstr(bfType)';
      kinf(i,10)=str2num(hstr);
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,18,'char');
      hstr=setstr(bfType)';
      kinf(i,11)=str2num(hstr);
      bfType=fread(fid,1,'int16');
      kinf(i,12)=bfType;
      bfType=fread(fid,1,'int16');
      kinf(i,13)=bfType;
      bfType=fread(fid,1,'int16');
      kinf(i,14)=bfType;
      bfType=fread(fid,1,'int16');
      kinf(i,15)=bfType;
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,8,'char');
      bfType=bfType';
      if bfType==[0,0,0,0,0,0,0,0]          %Rudi
        if i<10
            hs='  ';
        else
           if i<100
              hs=' ';
           else
              hs='';
           end; 
        end;
        bfType=['Kanal' int2str(i) hs];     
      end;
      kinf(i,17:24)=bfType;
      %kinf(i,17:24)=bfType';
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,8,'char');
      kinf(i,25:32)=bfType';
      bfType=fread(fid,1,'int16');
      kinf(i,16)=bfType;
      bfType=fread(fid,1,'int8');
      bfType=fread(fid,16,'char');
      kinf(i,33:48)=bfType';
    end;
    ainf(3)=kinf(1,3);
    if filepos(3)>0 
      %ainf(5)=(filepos(2)-filepos(3))/14; 
      mlen=flen-filepos(3);
      for i=1:3
        if fileindex(i)==3 
          mlen=sfilepos(i+1)-sfilepos(i); 
        end
      end; 
      ainf(5)=mlen / 14; 
    else
      ainf(5)=0;
      minf='';
    end;
    if ainf(5)>0
      h=fseek(fid,filepos(3),-1);
      for i=1:ainf(5)
        bfType=fread(fid,1,'uint16');
        bfType=fread(fid,1,'int16');
        minf(i,1)=bfType;  
        bfType=fread(fid,1,'long');
        minf(i,2)=bfType';
        bfType=fread(fid,1,'long');
        bfType=fread(fid,1,'uint16');
        manz(minf(i,1))=manz(minf(i,1))+1;
        
      end;
    end;
   
   
   if (nargin==1)|(mode~=0)
   if (nargin==1)|(mode>9)
      start=0;            %Rudi   
      %start=1;
      ende=ainf(4);
   end   
   if mode==2
      ende=start+ende;    %Rudi
      %ende=start+ende-1;
      if ende>ainf(4)
         ende=ainf(4);
      end   
   end   
   if mode==3
      if start>ainf(5)|ende>ainf(5)|start<1|ende<1      %????
         fclose(fid)
         error('Falsche Markerangaben');
      end;
      hilfs=minf(start,2);
      start=hilfs;
      hilfe=minf(ende,2);
      ende=hilfe;
   end;   
   if mode==4
      if start>ainf(5)|ende>ainf(4)|start<1|ende<1     %????
         fclose(fid)
         error('Falsche Parameterangaben');
      end;
      hilfs=minf(start,2);
      start=hilfs;
      ende=start+ende-1;   %????
      if ende>ainf(4)
         ende=ainf(4);
      end   
   
   end;   
   
   if mode==5
      if zykl<=0 
         fclose(fid);
         error('Falsche Parameterangaben! Parameter Zykl zu klein');
      end;
         
      startanz=find(minf(:,1)==start);
      endeanz=find(minf(:,1)==ende);
      if (size(startanz,1)<zykl)|(size(endeanz,1)<zykl)
         fclose(fid);
         error('Falsche Parameterangaben! Parameter Zykl zu gro oder Marker existiert nicht');
      end;
      start=minf(startanz(zykl),2);
      ende=minf(endeanz(zykl),2);
   end;   
   
   if mode==6
      if zykl<=0 
         fclose(fid);
         error('Falsche Parameterangaben! Parameter Zykl zu klein');
      end;
         
      startanz=find(minf(:,1)==start);
      if (size(startanz,1)<zykl)
         fclose(fid);
         error('Falsche Parameterangaben! Parameter Zykl zu gro oder Marker existiert nicht');
      end;
      start=minf(startanz(zykl),2);
      %ende=start+ende-1; 
      ende=start+ende;                         %Rudi
      if ende>ainf(4)
         ende=ainf(4);
      end   
   
   end;   
   
   if start<0      %Rudi
      start=0;
   end;
   %if start<1    
      %start=1;
   %end;
   if start>ende
      hilfv=ende;
      ende=start;
      start=hilfv;
   end;   
      
      h=fseek(fid,filepos(1),-1);
      
      if (ainf(1)==1)|(ainf(1)==3)
        
        %Rudi  
        if ainf(1)==3
           h=fseek(fid,ainf(2)*(start)*4+filepos(1),-1); 
           dinf=fread(fid,[ainf(2),(ende-start)],'float32');
        else
           h=fseek(fid,ainf(2)*(start)*2+filepos(1),-1); 
           dinf=fread(fid,[ainf(2),(ende-start)],'int16');
        end; 
        dinf= flipud(rot90(dinf)); 
        kagr=kinf(1:ainf(2),9)./32767;
        for j=1:ainf(2)
            dinf(:,j)=dinf(:,j)*kagr(j);
        end;
        %%dinf=dinf.*kinf(1,9)/32767;  
        
        
        
        %h=fseek(fid,ainf(2)*start*2+filepos(1),-1);
        %for i=start:ende
          %bfType=fread(fid,ainf(2),'int16');
          %for j=1:ainf(2)
            %dinf(i-start+1,j)=bfType(j)*kinf(j,9)/32767;
          %end;
        %end;
      
     else if ainf(1)==2
        h=fseek(fid,ainf(2)*start*2+filepos(1),-1);
        for i=1:ainf(2)
          bfType=fread(fid,ainf(4),'int16');
          for j=start:ende
            dinf(j-start+1,i)=bfType(j)*kinf(i,9)/32767;
          end;
        end;
     elseif ainf(1)==3
        h=fseek(fid,ainf(2)*start*4+filepos(1),-1);
        for i=start:ende
           bfType=fread(fid,ainf(2),'float32');
          for j=1:ainf(2)
            dinf(i-start+1,j)=bfType(j);
          end;
        end;
     end;
     end;
    end;

  
  else	  %ATISA 2.1-File
    h=fseek(fid,16400,-1);
    if isempty(h)
      fclose(fid);
      ainf(1)=100;
      error('Error in file.');
    end;
    bfType=fread(fid,1,'int8');  
    ainf(1)=3;

    for i=1:16
      bfType=fread(fid,18,'char');
      kinf(i)=str2num(setstr(bfType)');
      bfType=fread(fid,1,'int8');
    end;
    for i=1:16
      bfType=fread(fid,18,'char');
      kinf(16+i)=str2num(setstr(bfType)');
      bfType=fread(fid,1,'int8');
    end;

    bfType=fread(fid,18,'char');
    ainf(2)=str2num(setstr(bfType)');
    bfType=fread(fid,1,'int8');
    bfType=fread(fid,18,'char');
    ainf(4)=str2num(setstr(bfType)');
    bfType=fread(fid,1,'int8');
    bfType=fread(fid,18,'char');
    ainf(3)=str2num(setstr(bfType)');
    bfType=fread(fid,1,'int8');
    ainf(5)=0;
    if (nargin==1)|(mode>2)
       start=1;
       ende=512;
    end   
    if mode==2
       ende=start+ende-1;
       if ende>512
          ende=512;
       end   
    end   
    if start<1
       start=1;
    end;
    if start>ende
       hilfv=ende;
       ende=start;
       start=hilfv;
    end;   
       
    h=fseek(fid,0,-1);
    if (nargin==1)|(mode~=0)
      
      for i=1:16
        bfType=fread(fid,512,'int16');
        for j=start:ende
          dinf(j-start+1,i)=bfType(j)*kinf(i)/32767;
        end;
      end;
    end;
  end
end;

fclose(fid);

if nargin>1
   if mode==0
      disp('Headerinformationen')
      disp('-------------------')
      disp(Strcat('Kanalzahl: ',num2str(ainf(2))))
      disp(Strcat('Abtastfrequenz in Hz: ',num2str(ainf(3))))
      disp(Strcat('Mezyklen: ',num2str(ainf(4))))
      disp(Strcat('Markeranzahl: ',num2str(ainf(5))))
      if ainf(5)>0
         disp(' ')
         disp('Markerinformationen')
         disp('-------------------')
         disp(' ');
         disp(['Marker: ' '   Nummer'  '   Typ' '   Position (MW)' '    Position (ms)'])
         for i=1:ainf(5)
            disp(['Marker: ' '     ' num2str(i) '       ' num2str(minf(i,1)) '      ' num2str(minf(i,2)) '             ' num2str(1000*minf(i,2)/(ainf(3)))])
         end
         disp(' ')
         disp('Markerhufigkeit: ')
         disp('-----------------') 
         hm=find(manz(:)~=0);
         for i=1:size(hm,1)
           disp(['Markertyp: ' '     ' num2str(hm(i)) '      Anzahl   ' num2str(manz(hm(i)))]) 
         end   
      end   
   end;   
end;


